import { __decorate } from "tslib";
import { LifecycleMixin } from '@genesislcap/foundation-utils';
import { customElement, FoundationElement, attr } from '@genesislcap/web-core';
import { first, Subscription } from 'rxjs';
import { FDC3 } from '../fdc3';
import { logger } from '../utils';
/**
 * The Grid Pro element.
 * @remarks Grid Pro is a Web Component wrapper around the AG Grid Community library.
 * @public
 * @tagname %%prefix%%-grid-pro
 */
/**
 * fdc3-system-channel-listener
 * Util element to listen to the context on a fdc3 system channel
 * @public
 * @tagname fdc3-system-channel-listener
 * @example
 * ```ts
 * import { customElement, GenesisElement, html } from '@genesislcap/web-core';
 * @customElement({
 *   name: 'example-component',
 *   template: html<ExampleComponent>`
 *    <fdc3-system-channel-listener
 *      @channel-event="${(x, ctx) => x.handleChannelMessage(customEvent(ctx))}"
 *      context-type="fdc3.position">
 *    </fdc3-system-channel-listener>
 *   `,
 * })
 * class ExampleComponent extends GenesisElement {
 *   handleChannelMessage(e: CustomEvent) {
 *     const message = e.detail;
 *     if (message?.id?.ticker) {
 *       console.log(message.id.ticker);
 *       // ToDo: handle the message as you like
 *     }
 *   }
 * }
 * ```
 * @param Context - the fdc3 context type
 * @fires channel-event - Dispatched when an event matching the context is dispatched from the system channel
 *
 * @param Context - the fdc3 context type
 * @fires current-context - The current context value of the channel, if present
 * */
let Fdc3SystemChannelListener = class Fdc3SystemChannelListener extends LifecycleMixin(FoundationElement) {
    constructor() {
        super(...arguments);
        /**
         * @internal
         */
        this.channelSub = new Subscription();
    }
    connectedCallback() {
        super.connectedCallback();
        this.fdc3.isReady$.pipe(first()).subscribe(() => {
            this.addChannelListener();
        });
    }
    addChannelListener() {
        if (!this.contextType) {
            logger.warn('Fdc3SystemChannelListener: no context type provided');
        }
        else {
            this.channelSub.add(this.fdc3.currentChannel$.subscribe((channel) => {
                channel.addContextListener(this.contextType, (context) => {
                    this.$emit('channel-event', context);
                });
                this.handleCurrentContext(channel);
            }));
        }
    }
    handleCurrentContext(channel) {
        this.fdc3.handleChannelCurrentContext(channel, this.contextType, (context) => {
            if (context) {
                this.$emit('current-context', context);
            }
        });
    }
    disconnectedCallback() {
        var _a;
        (_a = this.channelSub) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        super.disconnectedCallback();
    }
};
__decorate([
    attr({ attribute: 'context-type' })
], Fdc3SystemChannelListener.prototype, "contextType", void 0);
__decorate([
    FDC3
], Fdc3SystemChannelListener.prototype, "fdc3", void 0);
Fdc3SystemChannelListener = __decorate([
    customElement({
        name: 'fdc3-system-channel-listener',
    })
], Fdc3SystemChannelListener);
export { Fdc3SystemChannelListener };
